<?php

require_once __DIR__ . '/../../../const.php';

use Bitrix\Main\Web\Json;
use Bitrix\Sale\BasketBase;
use Bitrix\Sale\Order;
use Bitrix\Sale\OrderBase;

IncludeModuleLangFile(__DIR__ . '/ferma-api.php');

class CFermaApiService
{
    /**
     * @var string
     */
    private $apiHost;
    /**
     * @var string
     */
    private $apiLogin;
    /**
     * @var string
     */
    private $apiPassword;
    /**
     * @var string
     */
    private $agentInn;
    /**
     * @var string
     */
    private $agentTaxation;
    /**
     * @var string
     */
    private $accessToken;
    /**
     * @var \CFermaApiClient
     */
    private $client;
    /**
     * @var bool
     */
    private $logJson = false;

    private static $instances = [];

    public function __construct(
        $apiHost,
        $apiLogin,
        $apiPassword,
        $agentInn,
        $agentTaxation,
        $errorsEmail,
        $timeout = 15
    )
    {
        $this->apiHost = $apiHost;
        $this->apiLogin = $apiLogin;
        $this->apiPassword = $apiPassword;
        $this->agentInn = $agentInn;
        $this->agentTaxation = $agentTaxation;

        $this->client = CFermaApiClient::instance($apiHost, $errorsEmail, $timeout);

        $this->logJson = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'receipt_json_log', 'N') === 'Y';
    }

    /**
     * @param string $apiHost
     * @param string $apiLogin
     * @param string $apiPassword
     * @param string $agentInn
     * @param string $agentTaxation
     * @param string $errorsEmail
     * @return CFermaApiClient
     */
    public static function instance($apiHost, $apiLogin, $apiPassword, $agentInn, $agentTaxation, $errorsEmail)
    {
        $hash = sha1($apiHost . '/' . $apiLogin . '/' . $apiPassword);

        if (!isset(static::$instances[$hash])) {
            static::$instances[$hash] = new static($apiHost, $apiLogin, $apiPassword, $agentInn, $agentTaxation, $errorsEmail);
        }

        return static::$instances[$hash];
    }

    public function getEnvId()
    {
        switch ($this->apiHost) {
            case 'https://ferma.ofd.ru':
                return 'prod';
            case 'https://ferma-test.ofd.ru':
                return 'demo';
            default:
                return null;
        }
    }

    public function getAccessToken()
    {
        if (null === $this->accessToken) {
            $this->renewAccessToken();
        }

        return $this->accessToken;
    }

    /**
     * @param string $receiptId
     * @return array
     * @throws \CFermaApiException
     */
    public function getElectronReceiptStatus($receiptId)
    {
        return $this->client->post('/api/kkt/cloud/status', ['Request' => ['ReceiptId' => $receiptId]], $this->getAccessToken());
    }

    /**
     * @param array $electronReceiptData
     * @return array
     * @throws \CFermaApiException
     */
    public function sendElectronReceipt($electronReceiptData)
    {
        if ($this->logJson) {
            CMainOfdFerma::log(
                GetMessage('FERMAOFD_SUBMIT_RECEIPT_TRY'), null,
                Json::encode($electronReceiptData, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT), 'DEBUG'
            );
        }

        return $this->client->post('/api/kkt/cloud/receipt', ['Request' => $electronReceiptData], $this->getAccessToken());
    }

    public function prepareElectronReceipt($type, Order $order)
    {
        list($email, $phone) = Ferma_getCustomerContacts($order->getId());

        $deliveryInfo = Ferma_getDeliveryInfo($order);

        return $this->createElectronReceipt(
            $type, $email, $phone,
            1, //
            $order->getId(),
            $order->getField('DATE_PAYED'),
            $this->formatOrderItems($order->getBasket(), $deliveryInfo)
        );
    }

    public static function createInvoiceId($orderId, $type)
    {
        // unique invoice id
        /** @noinspection NonSecureUniqidUsageInspection */
        return $orderId . '/' . $type . '/' . uniqid();
    }

    public function createElectronReceipt(
        $type,
        $email,
        $phone,
        $paymentType,
        $orderId,
        $orderDate,
        $items
    )
    {
        $date = new \DateTime;

        if ($type === 'Income') {
            $date = new \DateTime(empty($orderDate) ? 'now' : $orderDate);
        }

        return [
            'Inn' => $this->agentInn,
            'Type' => $type,
            'InvoiceId' => static::createInvoiceId($orderId, $type),
            'LocalDate' => $date->format('Y-m-d\TH:i:s'),
            'CustomerReceipt' => [
                'Email' => $email,
                'Phone' => $phone,
                'TaxationSystem' => $this->agentTaxation,
                'PaymentType' => $paymentType ? $paymentType : null,
                'Items' => array_values($items),
                'PaymentItems' => array_map(function ($item) {
                    return [
                        'PaymentType' => 1,
                        'Sum' => $item['Amount'],
                    ];
                }, $items),
            ],
        ];
    }

    private function formatOrderItems(BasketBase $basket, $deliveryInfo = null)
    {
        $result = [];

        $productsId = [];

        /** @var \Bitrix\Sale\BasketItem $item */
        foreach ($basket as $item) {
            $productsId[$item->getProductId()] = true;
        }

        if (empty($productsId)) {
            return $result;
        }

        $products = Ferma_getProducts(array_keys($productsId));
        $vats = Ferma_getVats();

        $defaultPaymentMethod = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'payment_default_method', '4');
        $defaultPaymentType = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'payment_default_type', '1');
        $defaultVat = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'payment_default_vat', 'VatNo');


        /** @var \Bitrix\Sale\BasketItem $item */
        foreach ($basket as $item) {
            $price = $item->getPrice();
            $quantity = $item->getQuantity();

            $vatId = $products[$item->getProductId()]['VAT_ID'];

            $paymentType = $products[$item->getProductId()]['PROPERTIES']['OFD_FERMA_PAYMENT_TYPE']['VALUE_XML_ID'];

            $result[] = [
                'Label' => $item->getField('NAME'),
                'Price' => $price,
                'Quantity' => $quantity,
                'Amount' => $price * $quantity,
                'Vat' => empty($vatId) ? $defaultVat : CVatCollection::determinateVat($vats[$vatId]),
                'PaymentMethod' => $defaultPaymentMethod,
                'PaymentType' => $paymentType ?: $defaultPaymentType,
            ];
        }

        $deliveryReceipt = COption::GetOptionString(FERMAOFD_FERMA_MODULE_ID, 'payment_delivery', 'Include');

        if ($deliveryReceipt === 'Include' && !empty($deliveryInfo) && !empty($deliveryInfo['PRICE'])) {
            $result[] = [
                'Label' => $deliveryInfo['NAME'],
                'Price' => $deliveryInfo['PRICE'],
                'Quantity' => 1,
                'Amount' => $deliveryInfo['PRICE'],
                'Vat' => $defaultVat,
                'PaymentMethod' => $defaultPaymentMethod,
                'PaymentType' => '4',
            ];
        }

        return $result;
    }

    private function renewAccessToken()
    {
        $response = $this->client->post('/api/Authorization/CreateAuthToken', [
            'Login' => $this->apiLogin,
            'Password' => $this->apiPassword,
        ]);

        if ($response['Status'] === 'Success') {
            $this->accessToken = $response['Data']['AuthToken'];
        } elseif ($response['Status'] === 'Failed' && isset($response['Error']['Message'])) {
            throw new CFermaApiException(sprintf('Api return failed response: %s.', $response['Error']['Message']));
        } else {
            throw new CFermaApiException('Api return invalid response.');
        }
    }
}

function Ferma_getProducts($productsId)
{
    CModule::IncludeModule('catalog');

    $result = [];

    $products = \Bitrix\Catalog\ProductTable::GetList([
        'select' => ['*'],
        'filter' => [
            '=ID' => $productsId,
        ],
    ]);

    $itemsId = [];
    $flatItemsId = [];
    $props = [];

    while ($product = $products->fetch()) {
        $result[$product['ID']] = $product;

        $data = CCatalogSku::GetProductInfo($product['ID']);

        if ($data) {
            if (!isset($itemsId[$data['IBLOCK_ID']])) {
                $itemsId[$data['IBLOCK_ID']] = [];
            }

            $itemsId[$data['IBLOCK_ID']][$product['ID']] = $data['ID'];
            $flatItemsId[$data['ID']] = $product['ID'];

            $props[$data['ID']] = [];
        }
    }

    foreach ($itemsId as $iBlockId => $map) {
        CIBlockElement::GetPropertyValuesArray($props, $iBlockId, [
            'ID' => array_values($map),
        ]);
    }

    foreach ($props as $itemId => $propsCollection) {
        $productId = $flatItemsId[$itemId];

        if (!isset($result[$productId])) {
            continue;
        }

        $result[$productId]['PROPERTIES'] = $propsCollection;
    }

    return $result;
}

function Ferma_getVats()
{
    $result = [];

    $vats = CCatalogVat::GetList();

    while ($product = $vats->NavNext()) {
        $result[$product['ID']] = $product;
    }

    return $result;
}

function Ferma_getCustomerContacts($orderId)
{
    $email = null;
    $phone = null;

    $props = (new CSaleOrderPropsValue())->GetList([], [
        'ORDER_ID' => $orderId,
        'CODE' => ['EMAIL', 'PHONE'],
    ]);

    while ($item = $props->Fetch()) {
        if ($item['CODE'] === 'EMAIL') {
            $email = $item['VALUE'];
        }

        if ($item['CODE'] === 'PHONE') {
            $phone = $item['VALUE'];
        }
    }

    return [$email, $phone];
}

function Ferma_getDeliveryInfo(OrderBase $order)
{
    $price = $order->getDeliveryPrice();

    if (empty($price)) {
        return null;
    }

    return [
        'NAME' => GetMessage('FERMAOFD_DELIVERY'),
        'PRICE' => $price,
    ];
}
